package com.docus.server.common;

import com.docus.core.util.DateUtil;
import com.docus.core.util.Func;
import com.docus.core.util.ListUtils;
import com.docus.core.util.json.JSON;
import com.docus.server.api.taskdistribute.TaskDistributeApi;
import com.docus.server.common.netty.CommMsg;
import com.docus.server.common.netty.server.ChannelRepository;
import com.docus.server.convert.CommMsgConvert;
import com.docus.server.dto.scheduling.management.schcollector.task.ReportDownTwoDTO;
import com.docus.server.dto.scheduling.management.schcollector.task.ReportTaskTwoDTO;
import com.docus.server.dto.scheduling.management.schterminator.CommMsgDTO;
import com.docus.server.dto.scheduling.management.schterminator.NettyTerminatorDTO;
import com.docus.server.enums.BusyStateEnum;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
public class SchedulerTask {
    @Resource
    private ChannelRepository channelRepository;
    @Resource
    private TaskDistributeApi taskDistributeApi;

    //定时任务
    // 5 * * * * ? 在每分钟的5秒执行
    @Scheduled(cron = "0/1 * * * * ?")
    public void scheduleTask() {
        try {
            log.info("定时任务： 开始执行");

            //在线并且空闲的终端
            List<NettyTerminatorDTO> terminalList = channelRepository.getAvailTerminator();

            //没有可用的通道
            if (Func.isEmpty(terminalList)) {
                return;
            }

            List<ReportDownTwoDTO> reportDownTwoDTOList;

            List<List<ReportTaskTwoDTO>> taskInfos = new ArrayList<>();

            Map<String, ReportDownTwoDTO> reportDownTwoMap = Maps.newConcurrentMap();
            if (!CollectionUtils.isEmpty(terminalList)) {
                //:todo 任务平台需要修改发布任务策略
                reportDownTwoDTOList = getTask(terminalList.size());

                if (Func.isEmpty(reportDownTwoDTOList) || Func.isBlank(reportDownTwoDTOList.get(0).getPatientId())) {
                    return;
                }

                reportDownTwoMap = ListUtils.toMap(reportDownTwoDTOList, ReportDownTwoDTO::getPatientId);

                taskInfos = ListUtils.select(reportDownTwoDTOList, ReportDownTwoDTO::getTasks);

            }

            //只采集，有优先级的
            for (NettyTerminatorDTO terminal : terminalList) {
                for (List<ReportTaskTwoDTO> taskInfo : taskInfos) {
                    //先找出有只采集的任务。
                    ReportTaskTwoDTO reportTaskTwoDto = taskInfo.get(0);
                    if (terminal.getOnlyCollectorIds().contains(reportTaskTwoDto.getCollectorId())
                        && BusyStateEnum.IDLE.equals(terminal.getBusyState())) {
                        //把这个任务派给这个终端,并且把这个终端设置成繁忙
                        if (terminal.getPriorityCollectorIds().contains(reportTaskTwoDto.getCollectorId())) {
                            //把这个任务派给这个终端
                            terminal.setBusyState(BusyStateEnum.BUSY);
                            String patientId = reportTaskTwoDto.getPatientId();
                            if (reportDownTwoMap.containsKey(patientId)) {
                                ReportDownTwoDTO reportDownTwoDTO = reportDownTwoMap.get(patientId);

                                tcpToClient(terminal, reportDownTwoDTO);
                            }
                            return;
                        }
                    }
                }
            }

            //只采集没有优先级
            for (NettyTerminatorDTO terminal : terminalList) {
                //把刚才已经分配任务过的采集器排除
                if (BusyStateEnum.BUSY.equals(terminal.getBusyState())) {
                    continue;
                }

                for (List<ReportTaskTwoDTO> taskInfo : taskInfos) {
                    //先找出有只采集的任务。
                    ReportTaskTwoDTO reportTaskTwoDto = taskInfo.get(0);
                    if (terminal.getOnlyCollectorIds().contains(reportTaskTwoDto.getCollectorId())
                        && BusyStateEnum.IDLE.equals(terminal.getBusyState())) {
                        //把这个任务派给这个终端,并且把这个终端设置成繁忙
                        terminal.setBusyState(BusyStateEnum.BUSY);
                        String patientId = reportTaskTwoDto.getPatientId();
                        if (reportDownTwoMap.containsKey(patientId)) {
                            ReportDownTwoDTO reportDownTwoDTO = reportDownTwoMap.get(patientId);
                            tcpToClient(terminal, reportDownTwoDTO);
                        }
                        return;
                    }
                }
            }


            //无只采集，有优先级
            for (NettyTerminatorDTO terminal : terminalList) {

                //把刚才已经分配任务过的采集器排除
                if (BusyStateEnum.BUSY.equals(terminal.getBusyState())) {
                    continue;
                }

                for (List<ReportTaskTwoDTO> taskInfo : taskInfos) {
                    //先找出有只采集的任务。
                    ReportTaskTwoDTO reportTaskTwoDto = taskInfo.get(0);
                    if (terminal.getPriorityCollectorIds().contains(reportTaskTwoDto.getCollectorId())
                        && BusyStateEnum.IDLE.equals(terminal.getBusyState())) {
                        //把这个任务派给这个终端
                        terminal.setBusyState(BusyStateEnum.BUSY);
                        String patientId = reportTaskTwoDto.getPatientId();
                        if (reportDownTwoMap.containsKey(patientId)) {
                            ReportDownTwoDTO reportDownTwoDTO = reportDownTwoMap.get(patientId);
                            tcpToClient(terminal, reportDownTwoDTO);
                        }
                        return;
                    }
                }
            }


            //无只采集，无优先级
            for (NettyTerminatorDTO terminal : terminalList) {

                //把刚才已经分配任务过的采集器排除
                if (BusyStateEnum.BUSY.equals(terminal.getBusyState())) {
                    continue;
                }

                for (List<ReportTaskTwoDTO> taskInfo : taskInfos) {
                    //先找出有只采集的任务。
                    //把这个任务派给这个终端
                    ReportTaskTwoDTO reportTaskTwoDto = taskInfo.get(0);
                    String collectorId = reportTaskTwoDto.getCollectorId();
                    terminal.setBusyState(BusyStateEnum.BUSY);
                    String patientId = reportTaskTwoDto.getPatientId();
                    if (reportDownTwoMap.containsKey(patientId)) {
                        ReportDownTwoDTO reportDownTwoDTO = reportDownTwoMap.get(patientId);
                        tcpToClient(terminal, reportDownTwoDTO);
                    }
                    return;
                }
            }


            log.info("定时任务： 执行完毕");
        } catch (Exception e) {
            log.error("定时任务执行出错", e);
        }
    }

    private void tcpToClient(NettyTerminatorDTO terminal, ReportDownTwoDTO messageContent) {
        Channel channel = channelRepository.get(terminal.getTerminatorIp());

        CommMsgDTO commMsgDTO = CommMsgDTO.builder()
            .content(JSON.toJSON(messageContent))
            .messageType(MsgConstants.SCH_DISTRIBUTE_TASKS)
            .build();

        CommMsg commMsg = CommMsgConvert.INSTANCE.convertDO(commMsgDTO);

        commMsg.setMessageTime(DateUtil.formatDateTime(new Date()));

        if (channel != null) {
            channel.writeAndFlush(Unpooled.copiedBuffer(JSON.toJSON(commMsg), CharsetUtil.UTF_8));
        }
    }

    //根据有效终端，一次获取一批任务，例如10个终端，获取10个不同类型任务
    private List<ReportDownTwoDTO> getTask(int size) {
//        return taskDistributeApi.getTask(String.valueOf(size));

        //mock

        ReportDownTwoDTO task = taskDistributeApi.getTask("1");
        ReportDownTwoDTO task1 = taskDistributeApi.getTask("1");

        return Lists.newArrayList(task, task1);
    }

    //根据采集器id类型，一次获取一批采集器类型任务
    private ArrayList<ReportDownTwoDTO> getTask(List<String> collectorIds) {
//        return taskDistributeApi.getTask(collectorIds.get(0));

        //mock
        ReportDownTwoDTO task = taskDistributeApi.getTask("4");
        ReportDownTwoDTO task1 = taskDistributeApi.getTask("5");

        return Lists.newArrayList(task, task1);
    }
}

